window.Gleam.initSearch([{"doc":"glisten","title":"glisten","content":"# glisten\n\n[![Package Version](https://img.shields.io/hexpm/v/glisten)](https://hex.pm/packages/glisten)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/glisten/)\n\nA Gleam TCP server library.  Built on top of `gleam_otp`, it provides a\nsupervisor over a pool of socket acceptors.  Each acceptor will block on `accept`\nuntil a connection is opened.  The acceptor will then spawn a handler process\nand then block again on `accept`.\n\nHere is a simple example that will echo received messages:\n\n```gleam\nimport gleam/bit_builder\nimport gleam/erlang/process\nimport gleam/option.{None}\nimport gleam/otp/actor\nimport glisten.{Packet}\n\npub fn main() {\n  let assert Ok(_) =\n    glisten.handler(\n      fn() { #(Nil, None) },\n      fn(msg, state, conn) {\n        let assert Packet(msg) = msg\n        let assert Ok(_) = glisten.send(conn, bit_builder.from_bit_string(msg))\n        actor.continue(state)\n      },\n    )\n    |&gt; glisten.serve(3000)\n\n  process.sleep_forever()\n}\n```\n\nSSL is also handled using the `glisten.{serve_ssl}` method.  This requires a\ncertificate and key file path.  The rest of the handler flow remains unchanged.\n\n`glisten` doesn&#39;t provide a public API for connected clients.  In order to hook\ninto the socket lifecyle, you can establish some functions which are called\nfor the opening and closing of the socket.  An example is provided below.\n\nTo serve over SSL:\n\n```gleam\n// ...\nimport glisten/ssl\n\npub fn main() {\n  let assert Ok(_) =\n    glisten.handler(\n      // omitted\n    )\n    |&gt; glisten.serve_ssl(\n      // Passing labeled arguments for clarity\n      port: 8080,\n      certfile: &quot;/path/to/server.crt&quot;,\n      keyfile: &quot;/path/to/server.key&quot;,\n    )\n  process.sleep_forever()\n}\n```\n\nBut you can also drop down to the lower level listen/accept flow if you&#39;d prefer\nto manage connections yourself, or only handle a small number at a time.\n\n```gleam\nimport gleam/io\nimport gleam/result\nimport glisten/socket/options.{ActiveMode, Passive}\nimport glisten/tcp\n\npub fn main() {\n  use listener &lt;- result.then(tcp.listen(8000, [ActiveMode(Passive)]))\n  use socket &lt;- result.then(tcp.accept(listener))\n  use msg &lt;- result.then(tcp.receive(socket, 0))\n  io.debug(#(&quot;got a msg&quot;, msg))\n\n  Ok(Nil)\n}\n```\n\nSee [mist](https://github.com/rawhat/mist) for HTTP support built on top of this library.\n","url":"index.html"},{"doc":"glisten/socket","title":"ListenSocket","content":"pub opaque type ListenSocket\n\n\nSynonyms:\nglisten/socket.ListenSocket\nglisten/socket ListenSocket","url":"glisten/socket.html#ListenSocket"},{"doc":"glisten/socket","title":"Socket","content":"pub opaque type Socket\n\n\nSynonyms:\nglisten/socket.Socket\nglisten/socket Socket","url":"glisten/socket.html#Socket"},{"doc":"glisten/socket","title":"SocketReason","content":"pub type SocketReason {\n  Closed\n  Timeout\n  Badarg\n  Terminated\n  Eaddrinuse\n  Eaddrnotavail\n  Eafnosupport\n  Ealready\n  Econnaborted\n  Econnrefused\n  Econnreset\n  Edestaddrreq\n  Ehostdown\n  Ehostunreach\n  Einprogress\n  Eisconn\n  Emsgsize\n  Enetdown\n  Enetunreach\n  Enopkg\n  Enoprotoopt\n  Enotconn\n  Enotty\n  Enotsock\n  Eproto\n  Eprotonosupport\n  Eprototype\n  Esocktnosupport\n  Etimedout\n  Ewouldblock\n  Exbadport\n  Exbadseq\n  Eacces\n  Eagain\n  Ebadf\n  Ebadmsg\n  Ebusy\n  Edeadlk\n  Edeadlock\n  Edquot\n  Eexist\n  Efault\n  Efbig\n  Eftype\n  Eintr\n  Einval\n  Eio\n  Eisdir\n  Eloop\n  Emfile\n  Emlink\n  Emultihop\n  Enametoolong\n  Enfile\n  Enobufs\n  Enodev\n  Enolck\n  Enolink\n  Enoent\n  Enomem\n  Enospc\n  Enosr\n  Enostr\n  Enosys\n  Enotblk\n  Enotdir\n  Enotsup\n  Enxio\n  Eopnotsupp\n  Eoverflow\n  Eperm\n  Epipe\n  Erange\n  Erofs\n  Espipe\n  Esrch\n  Estale\n  Etxtbsy\n  Exdev\n}\n\nClosed\n\n\nTimeout\n\n\nBadarg\n\n\nTerminated\n\n\nEaddrinuse\n\n\nEaddrnotavail\n\n\nEafnosupport\n\n\nEalready\n\n\nEconnaborted\n\n\nEconnrefused\n\n\nEconnreset\n\n\nEdestaddrreq\n\n\nEhostdown\n\n\nEhostunreach\n\n\nEinprogress\n\n\nEisconn\n\n\nEmsgsize\n\n\nEnetdown\n\n\nEnetunreach\n\n\nEnopkg\n\n\nEnoprotoopt\n\n\nEnotconn\n\n\nEnotty\n\n\nEnotsock\n\n\nEproto\n\n\nEprotonosupport\n\n\nEprototype\n\n\nEsocktnosupport\n\n\nEtimedout\n\n\nEwouldblock\n\n\nExbadport\n\n\nExbadseq\n\n\nEacces\n\n\nEagain\n\n\nEbadf\n\n\nEbadmsg\n\n\nEbusy\n\n\nEdeadlk\n\n\nEdeadlock\n\n\nEdquot\n\n\nEexist\n\n\nEfault\n\n\nEfbig\n\n\nEftype\n\n\nEintr\n\n\nEinval\n\n\nEio\n\n\nEisdir\n\n\nEloop\n\n\nEmfile\n\n\nEmlink\n\n\nEmultihop\n\n\nEnametoolong\n\n\nEnfile\n\n\nEnobufs\n\n\nEnodev\n\n\nEnolck\n\n\nEnolink\n\n\nEnoent\n\n\nEnomem\n\n\nEnospc\n\n\nEnosr\n\n\nEnostr\n\n\nEnosys\n\n\nEnotblk\n\n\nEnotdir\n\n\nEnotsup\n\n\nEnxio\n\n\nEopnotsupp\n\n\nEoverflow\n\n\nEperm\n\n\nEpipe\n\n\nErange\n\n\nErofs\n\n\nEspipe\n\n\nEsrch\n\n\nEstale\n\n\nEtxtbsy\n\n\nExdev\n\n\nSynonyms:\nglisten/socket.SocketReason\nglisten/socket SocketReason","url":"glisten/socket.html#SocketReason"},{"doc":"glisten/socket","title":"glisten/socket","content":"","url":"glisten/socket.html"},{"doc":"glisten/socket/options","title":"ActiveState","content":"pub type ActiveState {\n  Once\n  Passive\n  Count(Int)\n  Active\n}\n Mapping to the {active, _} option\nOnce\n\n\nPassive\n\n\nCount(Int)\n\n\nActive\n\n\nSynonyms:\nglisten/socket/options.ActiveState\nglisten/socket/options ActiveState","url":"glisten/socket/options.html#ActiveState"},{"doc":"glisten/socket/options","title":"SocketMode","content":"pub type SocketMode {\n  Binary\n}\n Mode for the socket.  Currently `list` is not supported\nBinary\n\n\nSynonyms:\nglisten/socket/options.SocketMode\nglisten/socket/options SocketMode","url":"glisten/socket/options.html#SocketMode"},{"doc":"glisten/socket/options","title":"TcpOption","content":"pub type TcpOption {\n  Backlog(Int)\n  Nodelay(Bool)\n  Linger(#(Bool, Int))\n  SendTimeout(Int)\n  SendTimeoutClose(Bool)\n  Reuseaddr(Bool)\n  ActiveMode(ActiveState)\n  Mode(SocketMode)\n  Certfile(String)\n  Keyfile(String)\n  AlpnPreferredProtocols(List(String))\n  Inet6\n}\n Options for the TCP socket\nBacklog(Int)\n\n\nNodelay(Bool)\n\n\nLinger(#(Bool, Int))\n\n\nSendTimeout(Int)\n\n\nSendTimeoutClose(Bool)\n\n\nReuseaddr(Bool)\n\n\nActiveMode(ActiveState)\n\n\nMode(SocketMode)\n\n\nCertfile(String)\n\n\nKeyfile(String)\n\n\nAlpnPreferredProtocols(List(String))\n\n\nInet6\n\n\nSynonyms:\nglisten/socket/options.TcpOption\nglisten/socket/options TcpOption","url":"glisten/socket/options.html#TcpOption"},{"doc":"glisten/socket/options","title":"merge_with_defaults","content":"pub fn merge_with_defaults(\n  options: List(TcpOption),\n) -&gt; List(TcpOption)\n\nSynonyms:\nglisten/socket/options.merge_with_defaults\nglisten/socket/options merge_with_defaults","url":"glisten/socket/options.html#merge_with_defaults"},{"doc":"glisten/socket/options","title":"to_map","content":"pub fn to_map(options: List(TcpOption)) -&gt; Map(Atom, Dynamic)\n\nSynonyms:\nglisten/socket/options.to_map\nglisten/socket/options to_map","url":"glisten/socket/options.html#to_map"},{"doc":"glisten/socket/options","title":"glisten/socket/options","content":"","url":"glisten/socket/options.html"},{"doc":"glisten/ssl","title":"accept","content":"pub fn accept(\n  socket: ListenSocket,\n) -&gt; Result(Socket, SocketReason)\n\nSynonyms:\nglisten/ssl.accept\nglisten/ssl accept","url":"glisten/ssl.html#accept"},{"doc":"glisten/ssl","title":"accept_timeout","content":"pub fn accept_timeout(\n  socket: ListenSocket,\n  timeout: Int,\n) -&gt; Result(Socket, SocketReason)\n\nSynonyms:\nglisten/ssl.accept_timeout\nglisten/ssl accept_timeout","url":"glisten/ssl.html#accept_timeout"},{"doc":"glisten/ssl","title":"close","content":"pub fn close(socket: Socket) -&gt; Result(Nil, SocketReason)\n\nSynonyms:\nglisten/ssl.close\nglisten/ssl close","url":"glisten/ssl.html#close"},{"doc":"glisten/ssl","title":"controlling_process","content":"pub fn controlling_process(\n  socket: Socket,\n  pid: Pid,\n) -&gt; Result(Nil, Atom)\n\nSynonyms:\nglisten/ssl.controlling_process\nglisten/ssl controlling_process","url":"glisten/ssl.html#controlling_process"},{"doc":"glisten/ssl","title":"do_shutdown","content":"pub fn do_shutdown(\n  socket: Socket,\n  write: Atom,\n) -&gt; Result(Nil, SocketReason)\n\nSynonyms:\nglisten/ssl.do_shutdown\nglisten/ssl do_shutdown","url":"glisten/ssl.html#do_shutdown"},{"doc":"glisten/ssl","title":"handshake","content":"pub fn handshake(socket: Socket) -&gt; Result(Socket, Nil)\n\nSynonyms:\nglisten/ssl.handshake\nglisten/ssl handshake","url":"glisten/ssl.html#handshake"},{"doc":"glisten/ssl","title":"listen","content":"pub fn listen(\n  port: Int,\n  options: List(TcpOption),\n) -&gt; Result(ListenSocket, SocketReason)\n Start listening over SSL on a port with the given options\nSynonyms:\nglisten/ssl.listen\nglisten/ssl listen","url":"glisten/ssl.html#listen"},{"doc":"glisten/ssl","title":"negotiated_protocol","content":"pub fn negotiated_protocol(\n  socket: Socket,\n) -&gt; Result(String, String)\n\nSynonyms:\nglisten/ssl.negotiated_protocol\nglisten/ssl negotiated_protocol","url":"glisten/ssl.html#negotiated_protocol"},{"doc":"glisten/ssl","title":"peername","content":"pub fn peername(\n  socket: Socket,\n) -&gt; Result(#(#(Int, Int, Int, Int), Int), Nil)\n\nSynonyms:\nglisten/ssl.peername\nglisten/ssl peername","url":"glisten/ssl.html#peername"},{"doc":"glisten/ssl","title":"receive","content":"pub fn receive(\n  socket: Socket,\n  length: Int,\n) -&gt; Result(BitArray, SocketReason)\n\nSynonyms:\nglisten/ssl.receive\nglisten/ssl receive","url":"glisten/ssl.html#receive"},{"doc":"glisten/ssl","title":"receive_timeout","content":"pub fn receive_timeout(\n  socket: Socket,\n  length: Int,\n  timeout: Int,\n) -&gt; Result(BitArray, SocketReason)\n\nSynonyms:\nglisten/ssl.receive_timeout\nglisten/ssl receive_timeout","url":"glisten/ssl.html#receive_timeout"},{"doc":"glisten/ssl","title":"send","content":"pub fn send(\n  socket: Socket,\n  packet: BytesBuilder,\n) -&gt; Result(Nil, SocketReason)\n\nSynonyms:\nglisten/ssl.send\nglisten/ssl send","url":"glisten/ssl.html#send"},{"doc":"glisten/ssl","title":"set_opts","content":"pub fn set_opts(\n  socket: Socket,\n  opts: List(TcpOption),\n) -&gt; Result(Nil, Nil)\n Update the optons for a socket (mutates the socket)\nSynonyms:\nglisten/ssl.set_opts\nglisten/ssl set_opts","url":"glisten/ssl.html#set_opts"},{"doc":"glisten/ssl","title":"shutdown","content":"pub fn shutdown(socket: Socket) -&gt; Result(Nil, SocketReason)\n\nSynonyms:\nglisten/ssl.shutdown\nglisten/ssl shutdown","url":"glisten/ssl.html#shutdown"},{"doc":"glisten/ssl","title":"start","content":"pub fn start() -&gt; Result(Nil, Dynamic)\n\nSynonyms:\nglisten/ssl.start\nglisten/ssl start","url":"glisten/ssl.html#start"},{"doc":"glisten/ssl","title":"glisten/ssl","content":"","url":"glisten/ssl.html"},{"doc":"glisten/tcp","title":"accept","content":"pub fn accept(\n  socket: ListenSocket,\n) -&gt; Result(Socket, SocketReason)\n\nSynonyms:\nglisten/tcp.accept\nglisten/tcp accept","url":"glisten/tcp.html#accept"},{"doc":"glisten/tcp","title":"accept_timeout","content":"pub fn accept_timeout(\n  socket: ListenSocket,\n  timeout: Int,\n) -&gt; Result(Socket, SocketReason)\n\nSynonyms:\nglisten/tcp.accept_timeout\nglisten/tcp accept_timeout","url":"glisten/tcp.html#accept_timeout"},{"doc":"glisten/tcp","title":"close","content":"pub fn close(socket: a) -&gt; Result(Nil, SocketReason)\n\nSynonyms:\nglisten/tcp.close\nglisten/tcp close","url":"glisten/tcp.html#close"},{"doc":"glisten/tcp","title":"controlling_process","content":"pub fn controlling_process(\n  socket: Socket,\n  pid: Pid,\n) -&gt; Result(Nil, Atom)\n\nSynonyms:\nglisten/tcp.controlling_process\nglisten/tcp controlling_process","url":"glisten/tcp.html#controlling_process"},{"doc":"glisten/tcp","title":"do_shutdown","content":"pub fn do_shutdown(\n  socket: Socket,\n  write: Atom,\n) -&gt; Result(Nil, SocketReason)\n\nSynonyms:\nglisten/tcp.do_shutdown\nglisten/tcp do_shutdown","url":"glisten/tcp.html#do_shutdown"},{"doc":"glisten/tcp","title":"handshake","content":"pub fn handshake(socket: Socket) -&gt; Result(Socket, Nil)\n\nSynonyms:\nglisten/tcp.handshake\nglisten/tcp handshake","url":"glisten/tcp.html#handshake"},{"doc":"glisten/tcp","title":"listen","content":"pub fn listen(\n  port: Int,\n  opts: List(TcpOption),\n) -&gt; Result(ListenSocket, SocketReason)\n Start listening over TCP on a port with the given options\nSynonyms:\nglisten/tcp.listen\nglisten/tcp listen","url":"glisten/tcp.html#listen"},{"doc":"glisten/tcp","title":"negotiated_protocol","content":"pub fn negotiated_protocol(socket: Socket) -&gt; a\n\nSynonyms:\nglisten/tcp.negotiated_protocol\nglisten/tcp negotiated_protocol","url":"glisten/tcp.html#negotiated_protocol"},{"doc":"glisten/tcp","title":"peername","content":"pub fn peername(\n  socket: Socket,\n) -&gt; Result(#(#(Int, Int, Int, Int), Int), Nil)\n\nSynonyms:\nglisten/tcp.peername\nglisten/tcp peername","url":"glisten/tcp.html#peername"},{"doc":"glisten/tcp","title":"receive","content":"pub fn receive(\n  socket: Socket,\n  length: Int,\n) -&gt; Result(BitArray, SocketReason)\n\nSynonyms:\nglisten/tcp.receive\nglisten/tcp receive","url":"glisten/tcp.html#receive"},{"doc":"glisten/tcp","title":"receive_timeout","content":"pub fn receive_timeout(\n  socket: Socket,\n  length: Int,\n  timeout: Int,\n) -&gt; Result(BitArray, SocketReason)\n\nSynonyms:\nglisten/tcp.receive_timeout\nglisten/tcp receive_timeout","url":"glisten/tcp.html#receive_timeout"},{"doc":"glisten/tcp","title":"send","content":"pub fn send(\n  socket: Socket,\n  packet: BytesBuilder,\n) -&gt; Result(Nil, SocketReason)\n\nSynonyms:\nglisten/tcp.send\nglisten/tcp send","url":"glisten/tcp.html#send"},{"doc":"glisten/tcp","title":"set_opts","content":"pub fn set_opts(\n  socket: Socket,\n  opts: List(TcpOption),\n) -&gt; Result(Nil, Nil)\n Update the optons for a socket (mutates the socket)\nSynonyms:\nglisten/tcp.set_opts\nglisten/tcp set_opts","url":"glisten/tcp.html#set_opts"},{"doc":"glisten/tcp","title":"shutdown","content":"pub fn shutdown(socket: Socket) -&gt; Result(Nil, SocketReason)\n\nSynonyms:\nglisten/tcp.shutdown\nglisten/tcp shutdown","url":"glisten/tcp.html#shutdown"},{"doc":"glisten/tcp","title":"socket_info","content":"pub fn socket_info(socket: Socket) -&gt; Map(a, b)\n\nSynonyms:\nglisten/tcp.socket_info\nglisten/tcp socket_info","url":"glisten/tcp.html#socket_info"},{"doc":"glisten/tcp","title":"glisten/tcp","content":"","url":"glisten/tcp.html"},{"doc":"glisten/socket/transport","title":"Transport","content":"pub type Transport {\n  Ssl(\n    accept: Accept,\n    accept_timeout: AcceptTimeout,\n    close: Close,\n    controlling_process: ControllingProcess,\n    handshake: Handshake,\n    listen: Listen,\n    negotiated_protocol: NegotiatedProtocol,\n    peername: PeerName,\n    receive: Receive,\n    receive_timeout: ReceiveTimeout,\n    send: Send,\n    set_opts: SetOpts,\n    shutdown: Shutdown,\n    socket_info: SocketInfo,\n  )\n  Tcp(\n    accept: Accept,\n    accept_timeout: AcceptTimeout,\n    close: Close,\n    controlling_process: ControllingProcess,\n    handshake: Handshake,\n    listen: Listen,\n    negotiated_protocol: NegotiatedProtocol,\n    peername: PeerName,\n    receive: Receive,\n    receive_timeout: ReceiveTimeout,\n    send: Send,\n    set_opts: SetOpts,\n    shutdown: Shutdown,\n    socket_info: SocketInfo,\n  )\n}\n\nSsl(\n  accept: Accept,\n  accept_timeout: AcceptTimeout,\n  close: Close,\n  controlling_process: ControllingProcess,\n  handshake: Handshake,\n  listen: Listen,\n  negotiated_protocol: NegotiatedProtocol,\n  peername: PeerName,\n  receive: Receive,\n  receive_timeout: ReceiveTimeout,\n  send: Send,\n  set_opts: SetOpts,\n  shutdown: Shutdown,\n  socket_info: SocketInfo,\n)\n\n\nTcp(\n  accept: Accept,\n  accept_timeout: AcceptTimeout,\n  close: Close,\n  controlling_process: ControllingProcess,\n  handshake: Handshake,\n  listen: Listen,\n  negotiated_protocol: NegotiatedProtocol,\n  peername: PeerName,\n  receive: Receive,\n  receive_timeout: ReceiveTimeout,\n  send: Send,\n  set_opts: SetOpts,\n  shutdown: Shutdown,\n  socket_info: SocketInfo,\n)\n\n\nSynonyms:\nglisten/socket/transport.Transport\nglisten/socket/transport Transport","url":"glisten/socket/transport.html#Transport"},{"doc":"glisten/socket/transport","title":"socket_info","content":"pub fn socket_info(socket: Socket) -&gt; Map(a, b)\n\nSynonyms:\nglisten/socket/transport.socket_info\nglisten/socket/transport socket_info","url":"glisten/socket/transport.html#socket_info"},{"doc":"glisten/socket/transport","title":"ssl","content":"pub fn ssl() -&gt; Transport\n\nSynonyms:\nglisten/socket/transport.ssl\nglisten/socket/transport ssl","url":"glisten/socket/transport.html#ssl"},{"doc":"glisten/socket/transport","title":"tcp","content":"pub fn tcp() -&gt; Transport\n\nSynonyms:\nglisten/socket/transport.tcp\nglisten/socket/transport tcp","url":"glisten/socket/transport.html#tcp"},{"doc":"glisten/socket/transport","title":"glisten/socket/transport","content":"","url":"glisten/socket/transport.html"},{"doc":"glisten/handler","title":"ClientIp","content":"pub type ClientIp =\n  Result(#(#(Int, Int, Int, Int), Int), Nil)\n\n\nSynonyms:\nglisten/handler.ClientIp\nglisten/handler ClientIp","url":"glisten/handler.html#ClientIp"},{"doc":"glisten/handler","title":"Connection","content":"pub type Connection(user_message) {\n  Connection(\n    client_ip: ClientIp,\n    socket: Socket,\n    transport: Transport,\n    sender: Subject(Message(user_message)),\n  )\n}\n\nConnection(\n  client_ip: ClientIp,\n  socket: Socket,\n  transport: Transport,\n  sender: Subject(Message(user_message)),\n)\n\n\nSynonyms:\nglisten/handler.Connection\nglisten/handler Connection","url":"glisten/handler.html#Connection"},{"doc":"glisten/handler","title":"Handler","content":"pub type Handler(user_message, data) {\n  Handler(\n    socket: Socket,\n    loop: Loop(user_message, data),\n    on_init: fn() -&gt; #(data, Option(Selector(user_message))),\n    on_close: Option(fn(data) -&gt; Nil),\n    transport: Transport,\n  )\n}\n\nHandler(\n  socket: Socket,\n  loop: Loop(user_message, data),\n  on_init: fn() -&gt; #(data, Option(Selector(user_message))),\n  on_close: Option(fn(data) -&gt; Nil),\n  transport: Transport,\n)\n\n\nSynonyms:\nglisten/handler.Handler\nglisten/handler Handler","url":"glisten/handler.html#Handler"},{"doc":"glisten/handler","title":"InternalMessage","content":"pub type InternalMessage {\n  Close\n  Ready\n  ReceiveMessage(BitArray)\n  Ssl(socket: Port, data: BitArray)\n  SslClosed\n  Tcp(socket: Port, data: BitArray)\n  TcpClosed\n}\n All message types that the handler will receive, or that you can\n send to the handler process\nClose\n\n\nReady\n\n\nReceiveMessage(BitArray)\n\n\nSsl(socket: Port, data: BitArray)\n\n\nSslClosed\n\n\nTcp(socket: Port, data: BitArray)\n\n\nTcpClosed\n\n\nSynonyms:\nglisten/handler.InternalMessage\nglisten/handler InternalMessage","url":"glisten/handler.html#InternalMessage"},{"doc":"glisten/handler","title":"Loop","content":"pub type Loop(user_message, data) =\n  fn(LoopMessage(user_message), data, Connection(user_message)) -&gt;\n    actor.Next(LoopMessage(user_message), data)\n\n\nSynonyms:\nglisten/handler.Loop\nglisten/handler Loop","url":"glisten/handler.html#Loop"},{"doc":"glisten/handler","title":"LoopMessage","content":"pub type LoopMessage(user_message) {\n  Packet(BitArray)\n  Custom(user_message)\n}\n\nPacket(BitArray)\n\n\nCustom(user_message)\n\n\nSynonyms:\nglisten/handler.LoopMessage\nglisten/handler LoopMessage","url":"glisten/handler.html#LoopMessage"},{"doc":"glisten/handler","title":"LoopState","content":"pub type LoopState(user_message, data) {\n  LoopState(\n    client_ip: ClientIp,\n    socket: Socket,\n    sender: Subject(Message(user_message)),\n    transport: Transport,\n    data: data,\n  )\n}\n\nLoopState(\n  client_ip: ClientIp,\n  socket: Socket,\n  sender: Subject(Message(user_message)),\n  transport: Transport,\n  data: data,\n)\n\n\nSynonyms:\nglisten/handler.LoopState\nglisten/handler LoopState","url":"glisten/handler.html#LoopState"},{"doc":"glisten/handler","title":"Message","content":"pub type Message(user_message) {\n  Internal(InternalMessage)\n  User(user_message)\n}\n\nInternal(InternalMessage)\n\n\nUser(user_message)\n\n\nSynonyms:\nglisten/handler.Message\nglisten/handler Message","url":"glisten/handler.html#Message"},{"doc":"glisten/handler","title":"start","content":"pub fn start(\n  handler: Handler(a, b),\n) -&gt; Result(Subject(Message(a)), StartError)\n Starts an actor for the TCP connection\nSynonyms:\nglisten/handler.start\nglisten/handler start","url":"glisten/handler.html#start"},{"doc":"glisten/handler","title":"glisten/handler","content":"","url":"glisten/handler.html"},{"doc":"glisten/logger","title":"error","content":"pub fn error(data: a) -&gt; Nil\n\nSynonyms:\nglisten/logger.error\nglisten/logger error","url":"glisten/logger.html#error"},{"doc":"glisten/logger","title":"glisten/logger","content":"","url":"glisten/logger.html"},{"doc":"glisten/acceptor","title":"AcceptorError","content":"pub type AcceptorError {\n  AcceptError\n  HandlerError\n  ControlError\n}\n\nAcceptError\n\n\nHandlerError\n\n\nControlError\n\n\nSynonyms:\nglisten/acceptor.AcceptorError\nglisten/acceptor AcceptorError","url":"glisten/acceptor.html#AcceptorError"},{"doc":"glisten/acceptor","title":"AcceptorMessage","content":"pub type AcceptorMessage {\n  AcceptConnection(ListenSocket)\n}\n\nAcceptConnection(ListenSocket)\n\n\nSynonyms:\nglisten/acceptor.AcceptorMessage\nglisten/acceptor AcceptorMessage","url":"glisten/acceptor.html#AcceptorMessage"},{"doc":"glisten/acceptor","title":"AcceptorState","content":"pub type AcceptorState {\n  AcceptorState(\n    sender: Subject(AcceptorMessage),\n    socket: Option(Socket),\n    transport: Transport,\n  )\n}\n\nAcceptorState(\n  sender: Subject(AcceptorMessage),\n  socket: Option(Socket),\n  transport: Transport,\n)\n\n\nSynonyms:\nglisten/acceptor.AcceptorState\nglisten/acceptor AcceptorState","url":"glisten/acceptor.html#AcceptorState"},{"doc":"glisten/acceptor","title":"Pool","content":"pub type Pool(user_message, data) {\n  Pool(\n    listener_socket: ListenSocket,\n    handler: Loop(user_message, data),\n    pool_count: Int,\n    on_init: fn() -&gt; #(data, Option(Selector(user_message))),\n    on_close: Option(fn(data) -&gt; Nil),\n    transport: Transport,\n  )\n}\n\nPool(\n  listener_socket: ListenSocket,\n  handler: Loop(user_message, data),\n  pool_count: Int,\n  on_init: fn() -&gt; #(data, Option(Selector(user_message))),\n  on_close: Option(fn(data) -&gt; Nil),\n  transport: Transport,\n)\n\n\nSynonyms:\nglisten/acceptor.Pool\nglisten/acceptor Pool","url":"glisten/acceptor.html#Pool"},{"doc":"glisten/acceptor","title":"start","content":"pub fn start(\n  pool: Pool(a, b),\n) -&gt; Result(Subject(AcceptorMessage), StartError)\n Worker process that handles `accept`ing connections and starts a new process\n which receives the messages from the socket\nSynonyms:\nglisten/acceptor.start\nglisten/acceptor start","url":"glisten/acceptor.html#start"},{"doc":"glisten/acceptor","title":"start_pool","content":"pub fn start_pool(\n  pool: Pool(a, b),\n) -&gt; Result(Subject(Message), StartError)\n Starts a pool of acceptors of size `pool_count`.\n\n Runs `loop_fn` on ever message received\nSynonyms:\nglisten/acceptor.start_pool\nglisten/acceptor start_pool","url":"glisten/acceptor.html#start_pool"},{"doc":"glisten/acceptor","title":"glisten/acceptor","content":"","url":"glisten/acceptor.html"},{"doc":"glisten","title":"Connection","content":"pub type Connection(user_message) {\n  Connection(\n    client_ip: ClientIp,\n    socket: Socket,\n    transport: Transport,\n    subject: Subject(handler.Message(user_message)),\n  )\n}\n This type holds useful bits of data for the active connection.\nConnection(\n  client_ip: ClientIp,\n  socket: Socket,\n  transport: Transport,\n  subject: Subject(handler.Message(user_message)),\n)\n\nclient_ip\n&lt;p&gt;This will be optionally a tuple for the IPv4 of the other end of the\nsocket&lt;/p&gt;\n\ntransport\n&lt;p&gt;This provides a uniform interface for both TCP and SSL methods.&lt;/p&gt;\n\nSynonyms:\nglisten.Connection\nglisten Connection","url":"glisten.html#Connection"},{"doc":"glisten","title":"Handler","content":"pub opaque type Handler(user_message, data)\n\n\nSynonyms:\nglisten.Handler\nglisten Handler","url":"glisten.html#Handler"},{"doc":"glisten","title":"Loop","content":"pub type Loop(user_message, data) =\n  fn(Message(user_message), data, Connection(user_message)) -&gt;\n    actor.Next(Message(user_message), data)\n This is the shape of the function you need to provide for the `handler`\n argument to `serve(_ssl)`.\n\nSynonyms:\nglisten.Loop\nglisten Loop","url":"glisten.html#Loop"},{"doc":"glisten","title":"Message","content":"pub type Message(user_message) {\n  Packet(BitArray)\n  User(user_message)\n}\n Your provided loop function with receive these message types as the\n first argument.\nPacket(BitArray)\n These are messages received from the socket\n\nUser(user_message)\n These are any messages received from the selector returned from `on_init`\n\nSynonyms:\nglisten.Message\nglisten Message","url":"glisten.html#Message"},{"doc":"glisten","title":"StartError","content":"pub type StartError {\n  ListenerClosed\n  ListenerTimeout\n  AcceptorTimeout\n  AcceptorFailed(process.ExitReason)\n  AcceptorCrashed(Dynamic)\n  SystemError(SocketReason)\n}\n Reasons that `serve` might fail\nListenerClosed\n\n\nListenerTimeout\n\n\nAcceptorTimeout\n\n\nAcceptorFailed(process.ExitReason)\n\n\nAcceptorCrashed(Dynamic)\n\n\nSystemError(SocketReason)\n\n\nSynonyms:\nglisten.StartError\nglisten StartError","url":"glisten.html#StartError"},{"doc":"glisten","title":"handler","content":"pub fn handler(\n  on_init: fn() -&gt; #(a, Option(Selector(b))),\n  loop: fn(Message(b), a, Connection(b)) -&gt; Next(Message(b), a),\n) -&gt; Handler(b, a)\n Create a new handler for each connection.  The required arguments mirror the\n `actor.start` API from `gleam_otp`.  The default pool is 10 accceptor\n processes.\nSynonyms:\nglisten.handler\nglisten handler","url":"glisten.html#handler"},{"doc":"glisten","title":"send","content":"pub fn send(\n  conn: Connection(a),\n  msg: BytesBuilder,\n) -&gt; Result(Nil, SocketReason)\n Sends a BitBuilder message over the socket using the active transport\nSynonyms:\nglisten.send\nglisten send","url":"glisten.html#send"},{"doc":"glisten","title":"serve","content":"pub fn serve(\n  handler: Handler(a, b),\n  port: Int,\n) -&gt; Result(Nil, StartError)\n Start the TCP server with the given handler on the provided port\nSynonyms:\nglisten.serve\nglisten serve","url":"glisten.html#serve"},{"doc":"glisten","title":"serve_ssl","content":"pub fn serve_ssl(\n  handler: Handler(a, b),\n  port port: Int,\n  certfile certfile: String,\n  keyfile keyfile: String,\n) -&gt; Result(Nil, StartError)\n Start the SSL server with the given handler on the provided port.  The key\n and cert files must be provided, valid, and readable by the current user.\nSynonyms:\nglisten.serve_ssl\nglisten serve_ssl","url":"glisten.html#serve_ssl"},{"doc":"glisten","title":"with_close","content":"pub fn with_close(\n  handler: Handler(a, b),\n  on_close: fn(b) -&gt; Nil,\n) -&gt; Handler(a, b)\n Adds a function to the handler to be called when the connection is closed.\nSynonyms:\nglisten.with_close\nglisten with_close","url":"glisten.html#with_close"},{"doc":"glisten","title":"with_pool_size","content":"pub fn with_pool_size(\n  handler: Handler(a, b),\n  size: Int,\n) -&gt; Handler(a, b)\n Modify the size of the acceptor pool\nSynonyms:\nglisten.with_pool_size\nglisten with_pool_size","url":"glisten.html#with_pool_size"},{"doc":"glisten","title":"glisten","content":"","url":"glisten.html"}]);